//!parent
//!encoding=utf-8

//****************************************************************************
// @file      LSW CORE LIB: REGEXP
// @author    Valentin Schmidt
// @version   0.7
//****************************************************************************

/**
 * Escapes string to be for regexp usage
 * @param {string}
 * @return {number}
 */
function escape (me, str){
  return str.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&');
}

/**
 * Executes a search for a match in a specified string, returns list or VOID.
 * @param {string} patt - The RegEx pattern (as string)
 * @param {string} str - The input string
 * @param {string} [mod='g'] - Modifier string
 * @return {number} The position of first occurence (or -1)
 */
function exec (me, patt, str, mod){
  if (mod==null) mod='g';
  var re = new RegExp(patt, mod);
  var arr = re.exec(str);
  if (arr){
    var tList = list();
    for (var i in arr) tList.add(arr[i]);
    return tList;
  }
}

/**
 * Tests for match of pattern in string, returns true or false
 * @param {string} patt - The RegEx pattern (as string)
 * @param {string} str - The input string
 * @param {string} [mod='g'] - Modifier string
 * @return {bool}
 */
function test (me, patt, str, mod){
  if (mod==null) mod='g';
  var re = new RegExp(patt, mod);
  return re.test(str);
}

/**
 * Searches for first occurence of pattern in string, returns position
 * @param {string} patt - The RegEx pattern (as string)
 * @param {string} str - The input string
 * @param {string} [mod='g'] - Modifier string
 * @return {number} The position of first occurence (or -1)
 */
function search_pos (me, patt, str, mod){
  if (mod==null) mod='g';
  return str.search(new RegExp(patt, mod));
}

/**
 * Splits string according to pattern, returns list
 * @param {string} patt - The RegEx pattern (as string)
 * @param {string} str - The input string
 * @param {string} [mod='g'] - Modifier string
 * @return {list} Lingo list
 */
function split (me, patt, str, mod){
  if (mod==null) mod='g';
  var arr = str.split(new RegExp(patt, mod));
  var tList = list();
  for (var i in arr) tList.add(arr[i]);
  return tList;
}

/**
 * Retrieves the matches when matching a string against a regular expression.
 * @param {string} patt - The RegEx pattern (as string)
 * @param {string} str - The input string
 * @param {string} [mod='g'] - Modifier string
 * @return {list} Lingo list
 */
function match (me, patt, str, mod){
  if (mod==null) mod='g';
  var arr = str.match(new RegExp(patt, mod));
  var tList = list();
  for (var i in arr) tList.add(arr[i]);
  return tList;
}

/**
 * Retrieves matches for specfic capture when matching a string against a regular expression.
 * @param {string} patt - The RegEx pattern (as string)
 * @param {string} str - The input string
 * @param {string} [mod='g'] - Modifier string
 * @param {integer} [index=0] - Capture index
 * @return {list} Lingo list
 */
function match_index (me, patt, str, mod, index){
  if (mod==null) mod='g';
  if (index==null) index=0;
  var re = new RegExp(patt, mod)
  var arr;
  var tList = list();
  while ((arr = re.exec(str)) !== null) {
    tList.add(arr[index]);
  }
  return tList;
}

/**
 * Replaces pattern matches with fixed string
 * Not binary-safe!
 * @param {string} patt - The RegEx pattern (as string)
 * @param {string} replStr - The replace string
 * @param {string} str - The input string
 * @param {string} [mod='g'] - Modifier string
 * @return {string} The result string
 */
function replace (me, patt, replStr, str, mod){
  if (mod==null) mod='g';
  return str.replace(new RegExp(patt, mod), replStr);
}

/**
 * Replaces pattern matches with fixed string
 * Not binary-safe!
 * @param {string} pattList - List of RegEx patterns (as strings)
 * @param {string} replStrList - List of replace strings
 * @param {string} str - The input string
 * @param {string} [mod='g'] - Modifier string
 * @return {string} The result string
 */
function replace_multi (me, pattList, replStrList, str, mod){
  if (mod==null) mod='g';
  var i, len = pattList.count;
  for (i=1;i<=len;i++){
    str = str.replace(new RegExp(pattList[i], mod), replStrList[i]);
  }
  return str;
}

/**
 * Replaces pattern matches with strings specified by hash (JS object or Lingo propList)
 * Not binary-safe!
 * @param {string} patt - The RegEx pattern (as string)
 * @param {object|propList} hash - The replacement hash
 * @param {string} str - The input string
 * @param {string} [mod='g'] - Modifier string
 * @return {string} The result string
 */
function replace_hash (me, patt, hash, str, mod){
  if (mod==null) mod='g';
  return str.replace(new RegExp(patt, mod), function(c){return hash[c]});
}

/**
 * Replaces pattern matches with strings returned by specified callback
 * Not binary-safe!
 * @param {string} patt - The RegEx pattern (as string)
 * @param {string} str - The input string
 * @param {symbol} cbHandler - The callback function
 * @param {object} [cbObject=_movie]
 * @param {string} [mod='g'] - Modifier string
 * @return {string} [data] - custom data passed to callback
 */
function replace_callback (me, patt, str, cbHandler, cbObject, mod, data){
  if (!cbObject) cbObject=_movie;
  if (mod==null) mod='g';
  return str.replace(new RegExp(patt, mod), function(){
    var args = list();
    for (var i=0;i<arguments.length;i++) args.append(arguments[i]);
    return _movie.call(cbHandler, list(cbObject), args, data);
  });
}

/**
 * Searches for pattern in string
 * @param {string} patt - The RegEx pattern (as string)
 * @param {string} str - The input string
 * @param {integer} [ref] - Ref of capture group that should be returned (default = 0)
 * @param {list|null} [indexes] - If a list is passed, it's filled with indexes of search results
 * @param {string} [mod='g'] - Modifier string
 * @return {list} Found matches
 */
function search (me, patt, str, ref, indexes, mod){
  if (ref==null) ref=0;
  if (mod==null) mod='g';
  var re = new RegExp(patt, mod);
  var matches = list();
  if (typeof indexes=='object'){
    while ((aMatch = re.exec(str)) !== null) {
      matches.add(aMatch[ref]);
      indexes.add(aMatch.index);
    }
  }else{
    while ((aMatch = re.exec(str)) !== null) {
      matches.add(aMatch[ref]);
    }
  }
  return matches;
}

/**
 * Convert lower to upper case
 * @param {string} str - The input string
 * @return {string} The result string
 */
function strtoupper (me, str){
  return str.toUpperCase();
}

/**
 * Convert upper to lower case
 * @param {string} str - The input string
 * @return {string} The result string
 */
function strtolower (me, str){
  return str.toLowerCase();
}
